#!/bin/bash

INSTALL_DEPS=0

function usage() {
	[ -n "$@" ] && echo -e "$@\n" > /dev/stderr
	echo "Usage: $0 [-x] [-I]"
	echo "  -I/--install-deps   Install build dependencies"
	echo "  -x	            Enable debug mode"
	[ -n "$@" ] && exit 1
}

idx=1
while [ $idx -le $# ]
do
	OPT=${!idx}
	case $OPT in
		-I|--install-deps)
			INSTALL_DEPS=1
			;;
		-x)
			set -x
			;;
		*)
			usage "Unkown parameter '$OPT'"
	esac
	let idx=idx+1
done

# Enter source directory
cd $( dirname $0 )

# Install build dependencies
if [ $INSTALL_DEPS -eq 1 ]
then
	apt-get update
	apt-get install --no-install-recommends --yes git sed python3-all python3-dev python3-pip python3-venv dpkg-dev build-essential debhelper dh-python lsb-release wget jq
fi

# Create virtualvenv to install dependencies
if [ -d venv ]
then
	VENV=$( realpath venv )
	TEMP_VENV=0
else
	# Create a temporary venv
	VENV=$(mktemp -d)
	echo "Create a temporary virtualenv in $VENV to install build dependencies..."
	TEMP_VENV=1
	python3 -m venv $VENV
fi

# Install python build dependencies in virtualenv
# Note: to fix dephell problems:
# * install specific version of mistune, docutils
#   to fix m2r issues (https//github.com/miyakogi/m2r/issues/66)
# * install missing appdirs dependency
# * install version 21.3 of packaging because it's incompatible with newer release
$VENV/bin/python3 -m pip install mistune==0.8.4 docutils==0.18.1 packaging==21.3 appdirs
$VENV/bin/python3 -m pip install dephell stdeb GitPython tomlq poetry

# Check gitdch is installed
GITDCH=$(which gitdch)
set -e
if [ -z "$GITDCH" ]
then
	TMP_GITDCH=$(mktemp -d)
	echo "Temporary install gitdch in $TMP_GITDCH"
	wget -O $TMP_GITDCH/gitdch https://gitea.zionetrix.net/bn8/gitdch/raw/master/gitdch
	chmod +x $TMP_GITDCH/gitdch
	GITDCH=$TMP_GITDCH/gitdch
else
	TMP_GITDCH=""
fi

# Detect package name from pyproject.toml file
[ ! -e pyproject.toml ] && echo "pyproject.toml file not found. It's a poetry project?" && exit 1
PACKAGE_NAME="$( cat pyproject.toml|$VENV/bin/tomlq .tool.poetry.name -r 2> /dev/null|tr '_' '-')"
[ $? -ne 0 -o -z "$PACKAGE_NAME" ] && echo "Fail to detect package name from pyproject.toml file. It's a poetry project?" && exit 1

# Detect if argcomplete is used
USE_ARGCOMPLETE=$( cat pyproject.toml|$VENV/bin/tomlq -r '.tool.poetry.dependencies.argcomplete' 2> null )
[ -n "$USE_ARGCOMPLETE" -a "$USE_ARGCOMPLETE" != "null" ] && USE_ARGCOMPLETE=1 || USE_ARGCOMPLETE=0
[ $USE_ARGCOMPLETE -eq 1 -a $INSTALL_DEPS -eq 1 ] && apt-get install --no-install-recommends --yes bash-completion python3-argcomplete

# Install GPG key (if provided)
if [ -n "$GPG_KEY" ]
then
	[ $INSTALL_DEPS -eq 1 ] && apt-get install --no-install-recommends --yes gnupg2
	echo "$GPG_KEY"|base64 -d|gpg --import
fi

# Detect maintainer info from environment or pyproject.toml file
if [ -n "$DEBFULLNAME" ]
then
	MAINTAINER_NAME="$DEBFULLNAME"
else
	MAINTAINER_NAME="$( cat pyproject.toml|$VENV/bin/tomlq .tool.poetry.authors[0] -r 2> /dev/null|sed 's/^\(.*\) <.*$/\1/' )"
	[ $? -ne 0 -o -z "$MAINTAINER_NAME" ] && echo "Fail to detect maintainer name from pyproject.toml file. It's a poetry project?" && exit 1
fi

if [ -n "$DEBEMAIL" ]
then
	MAINTAINER_EMAIL="$DEBEMAIL"
else
	MAINTAINER_EMAIL="$( cat pyproject.toml|$VENV/bin/tomlq .tool.poetry.authors[0] -r 2> /dev/null|sed 's/^.*<\(.*\)>.*$/\1/' )"
	[ $? -ne 0 -o -z "$MAINTAINER_EMAIL" ] && echo "Fail to detect maintainer email from pyproject.toml file. It's a poetry project?" && exit 1
fi

# Clean previous build
rm -fr build *.egg-info *.tar.gz dist setup.py

# Compute version using git describe
VERSION="$( git describe --tags 2> /dev/null )"

# If no tag exist, git describe will fail: in this case, compute a 0.0 version with same format
# as git describe
[ $? -ne 0 ] && VERSION="0.0-$( git log|grep -cE ^'commit ' )-$( git describe --tags --always )"

# Fix version format to match with Python specs
# See: https://peps.python.org/pep-0440/
VERSION=$( echo "$VERSION"|sed 's/[^0-9]*\([0-9][^-]*\)-\(.*\)/\1+\2/' )

# Set version in pyproject.toml
sed -i "s/^version *=.*$/version = \"$VERSION\"/" pyproject.toml

# Build python whl package using poetry
POETRY_VIRTUALENVS_PATH=$VENV $VENV/bin/poetry build

# Compute EE debian codename
DEBIAN_RELEASE=$( lsb_release -r -s|sed 's/\..*$//' )
DEBIAN_CODENAME=$( lsb_release -c -s )
[ $DEBIAN_RELEASE -ge 9 ] && DEBIAN_CODENAME="${DEBIAN_CODENAME}-ee"

# Compute debian package version by adding EE debian version suffix
DEB_VERSION_SUFFIX="-1~ee${DEBIAN_RELEASE}0"
DEB_VERSION="${VERSION}${DEB_VERSION_SUFFIX}"

# Convert pyproject.toml to setup.py (using dephell)
$VENV/bin/dephell deps convert \
	--from-format poetry --to-format setuppy \
	--from-path pyproject.toml --to-path setup.py

# Build debian source package
$VENV/bin/python3 setup.py --command-packages=stdeb.command sdist_dsc \
	--package3 "$PACKAGE_NAME" \
	--maintainer "$MAINTAINER_NAME <$MAINTAINER_EMAIL>" \
	--compat 10 \
	--section net \
	--dist-dir dist \
	--forced-upstream-version "$VERSION"

# Keep only debian package directory and orig.tar.gz archive
find dist/ -maxdepth 1 -type f ! -name '*.orig.tar.gz' ! -name '*.whl' -delete

# Enter in debian package directory
cd dist/$PACKAGE_NAME-$VERSION

# Compute gitdch extra args
GITDCH_EXTRA_ARGS=()
[ -n "$DEBFULLNAME" ] && GITDCH_EXTRA_ARGS+=( "--maintainer-name" "$DEBFULLNAME" )
[ -n "$DEBEMAIL" ] && GITDCH_EXTRA_ARGS+=( "--maintainer-email" "$DEBEMAIL" )

# Generate debian changelog using generate_debian_changelog.py
$VENV/bin/python3 $GITDCH \
	--package-name $PACKAGE_NAME \
	--version="${DEB_VERSION}" \
	--version-suffix="${DEB_VERSION_SUFFIX}" \
	--code-name $DEBIAN_CODENAME \
	--output debian/changelog \
	--path ../../ \
	--release-notes ../release-notes.md \
	--exclude "^CI: " \
	--exclude "\.gitlab-ci\.yml" \
	--exclude "build\.sh" \
	--exclude "tests\.sh" \
	--exclude "README\.md" \
	--exclude "^Merge branch " \
	--verbose "${GITDCH_EXTRA_ARGS[@]}"

# Add custom package name for dependencies
# echo "mylib python3-mylib" > debian/py3dist-overrides

# Generate bash completion file
if [ $USE_ARGCOMPLETE ]
then
	# Enable debhelper bash-completion in debian/rules
	sed -i 's/\(dh \$@.*\)/\1 --with bash-completion/' debian/rules
	# Retreive _python_argcomplete* functions for a fake command to initialize bash-completion file
	FULL_RETURN=$( register-python-argcomplete3 -s bash fake-command )
	echo -e "$FULL_RETURN" | grep -v -E '^complete ' > debian/${PACKAGE_NAME}.bash-completion
	# Append a complete line for each scripts
	COMPLETE_LINE=$( echo -e "$FULL_RETURN" | grep -E '^complete ' )
	for script in $( cat pyproject.toml|$VENV/bin/tomlq -r '.tool.poetry.scripts | keys[]' )
	do
		echo $COMPLETE_LINE | sed "s/fake-command/$script/g" >> debian/${PACKAGE_NAME}.bash-completion
	done
fi

[ $TEMP_VENV -eq 1 ] && rm -fr $VENV

# Clean temporary gitdch installation
[ -n "$TMP_GITDCH" ] && rm -fr $TMP_GITDCH

# Build debian package
BUILD_ARGS=""
[ -z "$GPG_KEY" ] && BUILD_ARGS="--no-sign" || echo "GPG key provide, enable package signing."
dpkg-buildpackage $BUILD_ARGS
