#!/bin/bash

INSTALL_DEPS=0
JUNIT_OUTPUT=""

function usage() {
	[ -n "$@" ] && echo -e "$@\n" > /dev/stderr
	echo "Usage: $0 [-x] [--install-deps] [--junit-output output.xml]"
	echo "  -I/--install-deps   Install build dependencies"
	echo "  -J/--junit-output   Specify junit output file path"
	echo "  -x	            Enable debug mode"
	[ -n "$@" ] && exit 1
}

idx=1
while [ $idx -le $# ]
do
	OPT=${!idx}
	case $OPT in
		-I|--install-deps)
			INSTALL_DEPS=1
			;;
		-J|--junit-output)
			let idx=idx+1
			JUNIT_OUTPUT="${!idx}"
			;;
		-x)
			set -x
			;;
		*)
			usage "Unkown parameter '$OPT'"
	esac
	let idx=idx+1
done

# Enter source directory
cd $( dirname $0 )

# Install build dependencies
if [ $INSTALL_DEPS -eq 1 ]
then
	apt-get update
	apt-get install --no-install-recommends --yes python3-venv git
fi

# In junit output mode, clean previous generated reports
[ -n "$JUNIT_OUTPUT" ] && rm -f pytest.xml pylint.xml flake8.txt flake8.xml

if [ -d venv ]
then
	VENV=$( realpath venv )
	TEMP_VENV=0
else
	# Install dephell & stdeb in temporary venv
	VENV=$(mktemp -d)
	echo "Create a temporary virtualenv in $VENV to install build dependencies..."
	TEMP_VENV=1
	python3 -m venv $VENV
fi

$VENV/bin/python3 -m pip install poetry
export POETRY_VIRTUALENVS_PATH=$VENV
$VENV/bin/poetry install

ERROR=0

# Run pytest
PYTEST_ARGS=""
[ -n "$JUNIT_OUTPUT" ] && PYTEST_ARGS="--junitxml=pytest.xml"
$VENV/bin/poetry run pytest $PYTEST_ARGS tests
[ $? -ne 0 ] && ERROR=1

# Run pylint
PYLINT_ARGS=""
if [ -n "$JUNIT_OUTPUT" ]
then
	PYLINT_ARGS="--load-plugins=pylint_junit --output-format=junit --output pylint.xml"
	echo -n "Run pylint... "
fi
$VENV/bin/poetry run pylint --fail-under=9.5 $PYLINT_ARGS icinga1_objects_upgrader tests
if [ $? -ne 0 ]
then
	ERROR=1
	[ -n "$JUNIT_OUTPUT" ] && echo "ERROR"
elif [ -n "$JUNIT_OUTPUT" ]
then
	echo "done."
fi

# Run flake8
FLAKE8_ARGS=""
[ -n "$JUNIT_OUTPUT" ] && FLAKE8_ARGS="--output-file flake8.txt"
$VENV/bin/poetry run flake8 $FLAKE8_ARGS icinga1_objects_upgrader tests
[ $? -ne 0 ] && ERROR=1

if [ -n "$JUNIT_OUTPUT" ]
then
	$VENV/bin/poetry run flake8_junit flake8.txt flake8.xml
	echo

	echo -n "Generate $JUNIT_OUTPUT... "
	$VENV/bin/poetry run junitparser merge pytest.xml pylint.xml flake8.xml "$JUNIT_OUTPUT"
	[ $? -ne 0 ] && ( ERROR=1; echo "ERROR" ) || echo "done."
fi

unset POETRY_VIRTUALENVS_PATH

[ $TEMP_VENV -eq 1 ] && rm -fr $VENV

[ $ERROR -eq 1 ] && echo "ERROR - Some problems detected, see tests report."
exit $ERROR
